program ibguardinst;

{$apptype console}

uses sysutils, winsvc, windows;

const
  SERVICE_WIN32_OWN_PROCESS = $00000010;
  SERVICE_ERROR_CRITICAL    = $00000003;

  SERVICE_BOOT_START        = $00000000;
  SERVICE_SYSTEM_START      = $00000001;
  SERVICE_AUTO_START        = $00000002;
  SERVICE_DEMAND_START      = $00000003;
  SERVICE_DISABLED          = $00000004;


procedure InstallService(service_name, service_label, executable, startup_type: string);
var
  mgr: THandle;
  svc: THandle;
  sttyp: dword;
  stfoundflag: boolean;
begin
  if not FileExists(executable)
  then begin
    writeln('Executable file not found: '+executable);
    halt(1);
  end;

  stfoundflag:=false;
  startup_type:=uppercase(startup_type);

  if startup_type='AUTOMATIC' then begin
    sttyp:=SERVICE_AUTO_START;
    stfoundflag:=true;
  end;

  if startup_type='MANUAL' then begin
    sttyp:=SERVICE_DEMAND_START;
    stfoundflag:=true;
  end;

  if startup_type='DISABLED' then begin
    sttyp:=SERVICE_DISABLED;
    stfoundflag:=true;
  end;

  mgr:=OpenSCManager(nil,nil,SC_MANAGER_CREATE_SERVICE);
  if mgr<>0 then begin
    svc:=CreateService(mgr,PChar(service_name),PChar(service_label),
    SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS,
    sttyp,SERVICE_ERROR_CRITICAL,
    PChar(executable),PChar(''),nil,PChar(''),nil,PChar(''));
    if svc=0 then begin
      writeln('Returned NULL, Error: '+SysErrorMessage(GetLastError));
      halt(1);
    end else begin
      writeln('Service installed.');
      CloseServiceHandle(svc);
    end;
    CloseServiceHandle(mgr);
  end else begin
    writeln('Could not open Service Mgr. Error: '+SysErrorMessage(GetLastError));
  end;
end;


procedure RemoveService(service_name: string);
var
   mgr: THandle;
   svc: THandle;
begin
   mgr:=OpenSCManager(nil,nil,SC_MANAGER_CREATE_SERVICE);
   if mgr<>0 then begin
      svc:=OpenService(mgr,pchar(service_name),SERVICE_ALL_ACCESS);
      if svc<>0 then begin
         if DeleteService(svc) then begin
            writeln('Service removed.');
         end else begin
            writeln('Error: '+SysErrorMessage(GetLastError));
         end;
       end else begin
          writeln('Error: '+SysErrorMessage(GetLastError));
       end;
       CloseServiceHandle(mgr);
       end else begin
          writeln('Error: '+SysErrorMessage(GetLastError));
       end; 
end;

var
  flag: boolean;
begin

  writeln;
  writeln('ibguardinst (c) 2000,2001 by Tom White, version 2001.01.16.');
  writeln('Released under GPL.');
  writeln('  Firebird - the ultimate open source database server.');
  writeln;

  flag:=false;
  if uppercase(paramstr(1))='INSTALL' then begin
    if paramcount<>2 then begin
      writeln('Usage:');
      writeln('   ibguardinst INSTALL <path_to_executable>');
      writeln;
      halt(1);
    end;
    InstallService('InterBaseGuardian','InterBase Guardian',paramstr(2),'AUTOMATIC');
    flag:=true;
  end;

  if uppercase(paramstr(1))='REMOVE' then begin
    writeln('Service Name : InterBaseGuardian');
    RemoveService('InterBaseGuardian');
    flag:=true;
  end;

  if not flag then begin
    writeln('Usage:');
    writeln('ibguardinst INSTALL <path_to_executable>');
    writeln('ibguardinst REMOVE');
    writeln;
    writeln('INSTALL installs the service, pointing to the executable provided');
    writeln('REMOVE uninstalls the service.');
  end;
end.
